@@ -45,7 +45,7 @@ information will be user_dn or user_info.
 Additional classes can be found in the config module next to this one.
 """
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import unicode_literals
 
 import copy
 from functools import reduce
@@ -63,9 +63,12 @@ from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 import django.dispatch
 from django.utils import six
-from django.utils.encoding import force_str
+from django.utils.encoding import force_text
+from django.utils.inspect import func_supports_parameter
 
-from django_auth_ldap.config import ConfigurationWarning, LDAPGroupQuery, LDAPSearch, _LDAPConfig
+from django_auth_ldap.config import (
+    ConfigurationWarning, LDAPGroupQuery, LDAPSearch, _LDAPConfig,
+)
 
 
 logger = _LDAPConfig.get_logger()
@@ -141,9 +144,9 @@ class LDAPBackend(object):
     # The Django auth backend API
     #
 
-    def authenticate(self, request=None, username=None, password=None, **kwargs):
-        if bool(password) or self.settings.PERMIT_EMPTY_PASSWORD:
-            ldap_user = _LDAPUser(self, username=username.strip())
+    def authenticate(self, request, username=None, password=None, **kwargs):
+        if password or self.settings.PERMIT_EMPTY_PASSWORD:
+            ldap_user = _LDAPUser(self, username=username.strip(), request=request)
             user = self.authenticate_ldap_user(ldap_user, password)
         else:
             logger.debug('Rejecting empty password for {}'.format(username))
@@ -217,18 +220,6 @@ class LDAPBackend(object):
         The returned User object may be an unsaved model instance.
 
         """
-        if self.__class__.get_or_create_user != LDAPBackend.get_or_create_user:
-            # Older deprecated method overridden, defer to it instead.
-            warnings.warn(
-                "Method LDAPBacked.get_or_create_user() is deprecated and will "
-                "be removed in a future version. Override "
-                "LDAPBacked.get_or_build_user() instead. "
-                "LDAPBacked.get_or_build_user() does not need to save newly "
-                "built users.",
-                DeprecationWarning,
-            )
-            return self.get_or_create_user(username, ldap_user)
-
         model = self.get_user_model()
 
         if self.settings.USER_QUERY_FIELD:
@@ -250,26 +241,6 @@ class LDAPBackend(object):
 
         return (user, built)
 
-    def get_or_create_user(self, username, ldap_user):
-        """
-        This must return a (User, created) 2-tuple for the given LDAP user.
-        username is the Django-friendly username of the user. ldap_user.dn is
-        the user's DN and ldap_user.attrs contains all of their LDAP
-        attributes.
-
-        Deprecated, but remains for backwards compatibility. Remove in a future
-        release.
-        """
-        model = self.get_user_model()
-        username_field = model.USERNAME_FIELD
-
-        kwargs = {
-            username_field + '__iexact': username,
-            'defaults': {username_field: username.lower()}
-        }
-
-        return model.objects.get_or_create(**kwargs)
-
     def ldap_to_django_username(self, username):
         return username
 
@@ -305,7 +276,7 @@ class _LDAPUser(object):
     # Initialization
     #
 
-    def __init__(self, backend, username=None, user=None):
+    def __init__(self, backend, username=None, user=None, request=None):
         """
         A new LDAPUser must be initialized with either a username or an
         authenticated User object. If a user is given, the username will be
@@ -313,6 +284,7 @@ class _LDAPUser(object):
         """
         self.backend = backend
         self._username = username
+        self._request = request
 
         if user is not None:
             self._set_authenticated_user(user)
@@ -526,12 +498,15 @@ class _LDAPUser(object):
         if self._using_simple_bind_mode():
             self._user_dn = self._construct_simple_user_dn()
         else:
-            cache_key = valid_cache_key('django_auth_ldap.user_dn.{}'.format(self._username))
-            self._user_dn = cache.get_or_set(
-                cache_key,
-                self._search_for_user_dn,
-                self.settings.GROUP_CACHE_TIMEOUT
-            )
+            if self.settings.CACHE_TIMEOUT > 0:
+                cache_key = valid_cache_key('django_auth_ldap.user_dn.{}'.format(self._username))
+                self._user_dn = cache.get_or_set(
+                    cache_key,
+                    self._search_for_user_dn,
+                    self.settings.CACHE_TIMEOUT
+                )
+            else:
+                self._user_dn = self._search_for_user_dn()
 
     def _using_simple_bind_mode(self):
         return (self.settings.USER_DN_TEMPLATE is not None)
@@ -554,7 +529,7 @@ class _LDAPUser(object):
             raise ImproperlyConfigured('AUTH_LDAP_USER_SEARCH must be an LDAPSearch instance.')
 
         results = search.execute(self.connection, {'user': self._username})
-        if (results is not None) and (len(results) == 1):
+        if results is not None and len(results) == 1:
             (user_dn, self._user_attrs) = next(iter(results))
         else:
             user_dn = None
@@ -637,7 +612,7 @@ class _LDAPUser(object):
             self._user.save()
 
         # This has to wait until we're sure the user has a pk.
-        if bool(self.settings.MIRROR_GROUPS) or bool(self.settings.MIRROR_GROUPS_EXCEPT):
+        if self.settings.MIRROR_GROUPS or self.settings.MIRROR_GROUPS_EXCEPT:
             self._normalize_mirror_settings()
             self._mirror_groups()
 
@@ -718,7 +693,7 @@ class _LDAPUser(object):
 
             if not all(isinstance(value, six.string_types) for value in mge):
                 raise malformed_mirror_groups_except()
-            elif bool(mg):
+            elif mg:
                 warnings.warn(ConfigurationWarning("Ignoring {} in favor of {}".format(
                     self.settings._name("MIRROR_GROUPS"),
                     self.settings._name("MIRROR_GROUPS_EXCEPT")
@@ -819,8 +794,8 @@ class _LDAPUser(object):
         the life of this object. If False, then the caller only wishes to test
         the credentials, after which the connection will be considered unbound.
         """
-        self._get_connection().simple_bind_s(force_str(bind_dn),
-                                             force_str(bind_password))
+        self._get_connection().simple_bind_s(force_text(bind_dn),
+                                             force_text(bind_password))
 
         self._connection_bound = sticky
 
@@ -831,9 +806,19 @@ class _LDAPUser(object):
         if self._connection is None:
             uri = self.settings.SERVER_URI
             if callable(uri):
-                uri = uri()
+                if func_supports_parameter(uri, 'request'):
+                    uri = uri(self._request)
+                else:
+                    warnings.warn(
+                        'Update AUTH_LDAP_SERVER_URI callable %s.%s to accept '
+                        'a positional `request` argument. Support for callables '
+                        'accepting no arguments will be removed in a future '
+                        'version.' % (uri.__module__, uri.__name__),
+                        DeprecationWarning
+                    )
+                    uri = uri()
 
-            self._connection = self.backend.ldap.initialize(uri)
+            self._connection = self.backend.ldap.initialize(uri, bytes_mode=False)
 
             for opt, value in self.settings.CONNECTION_OPTIONS.items():
                 self._connection.set_option(opt, value)
@@ -938,16 +923,16 @@ class _LDAPUserGroups(object):
         return self._group_infos
 
     def _load_cached_attr(self, attr_name):
-        if self.settings.CACHE_GROUPS:
+        if self.settings.CACHE_TIMEOUT > 0:
             key = self._cache_key(attr_name)
             value = cache.get(key)
             setattr(self, attr_name, value)
 
     def _cache_attr(self, attr_name):
-        if self.settings.CACHE_GROUPS:
+        if self.settings.CACHE_TIMEOUT > 0:
             key = self._cache_key(attr_name)
             value = getattr(self, attr_name, None)
-            cache.set(key, value, self.settings.GROUP_CACHE_TIMEOUT)
+            cache.set(key, value, self.settings.CACHE_TIMEOUT)
 
     def _cache_key(self, attr_name):
         """
@@ -976,11 +961,10 @@ class LDAPSettings(object):
         'BIND_AS_AUTHENTICATING_USER': False,
         'BIND_DN': '',
         'BIND_PASSWORD': '',
-        'CACHE_GROUPS': False,
         'CONNECTION_OPTIONS': {},
         'DENY_GROUP': None,
         'FIND_GROUP_PERMS': False,
-        'GROUP_CACHE_TIMEOUT': None,
+        'CACHE_TIMEOUT': 0,
         'GROUP_SEARCH': None,
         'GROUP_TYPE': None,
         'MIRROR_GROUPS': None,
@@ -1010,6 +994,19 @@ class LDAPSettings(object):
             value = getattr(django.conf.settings, prefix + name, default)
             setattr(self, name, value)
 
+        # Compatibility with old caching settings.
+        if getattr(django.conf.settings, self._name('CACHE_GROUPS'), defaults.get('CACHE_GROUPS')):
+            warnings.warn(
+                'Found deprecated setting AUTH_LDAP_CACHE_GROUP. Use '
+                'AUTH_LDAP_CACHE_TIMEOUT instead.',
+                DeprecationWarning,
+            )
+            self.CACHE_TIMEOUT = getattr(
+                django.conf.settings,
+                self._name('GROUP_CACHE_TIMEOUT'),
+                defaults.get('GROUP_CACHE_TIMEOUT', 3600),
+            )
+
     def _name(self, suffix):
         return (self._prefix + suffix)
 
