@@ -29,14 +29,15 @@ Please see the docstring on the backend module for more information, including
 notes on naming conventions.
 """
 
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import unicode_literals
 
 import logging
 import pprint
 
 import ldap
+import ldap.filter
 
-from django.utils.encoding import force_str
+from django.utils.encoding import force_text
 from django.utils.tree import Node
 
 
@@ -48,7 +49,6 @@ class _LDAPConfig(object):
     """
     A private class that loads and caches some global objects.
     """
-    ldap = None
     logger = None
 
     _ldap_configured = False
@@ -56,23 +56,16 @@ class _LDAPConfig(object):
     @classmethod
     def get_ldap(cls, global_options=None):
         """
-        Returns the ldap module. The unit test harness will assign a mock object
-        to _LDAPConfig.ldap. It is imperative that the ldap module not be
-        imported anywhere else so that the unit tests will pass in the absence
-        of python-ldap.
+        Returns the configured ldap module.
         """
-        if cls.ldap is None:
-            import ldap.filter
-            cls.ldap = ldap
-
         # Apply global LDAP options once
-        if (not cls._ldap_configured) and (global_options is not None):
+        if not cls._ldap_configured and global_options is not None:
             for opt, value in global_options.items():
-                cls.ldap.set_option(opt, value)
+                ldap.set_option(opt, value)
 
             cls._ldap_configured = True
 
-        return cls.ldap
+        return ldap
 
     @classmethod
     def get_logger(cls):
@@ -153,9 +146,9 @@ class LDAPSearch(object):
 
         try:
             filterstr = self.filterstr % filterargs
-            results = connection.search_s(force_str(self.base_dn),
+            results = connection.search_s(force_text(self.base_dn),
                                           self.scope,
-                                          force_str(filterstr),
+                                          force_text(filterstr),
                                           self.attrlist)
         except ldap.LDAPError as e:
             results = []
@@ -182,7 +175,7 @@ class LDAPSearch(object):
         try:
             filterstr = self.filterstr % filterargs
             msgid = connection.search(
-                force_str(self.base_dn), self.scope, force_str(filterstr),
+                force_text(self.base_dn), self.scope, force_text(filterstr),
                 self.attrlist
             )
         except ldap.LDAPError as e:
@@ -431,14 +424,22 @@ class PosixGroupType(LDAPGroupType):
             user_uid = ldap_user.attrs['uid'][0]
 
             try:
-                is_member = ldap_user.connection.compare_s(force_str(group_dn), 'memberUid', force_str(user_uid))
+                is_member = ldap_user.connection.compare_s(
+                    force_text(group_dn),
+                    'memberUid',
+                    user_uid.encode('utf-8'),
+                )
             except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
                 is_member = False
 
             if not is_member:
                 try:
                     user_gid = ldap_user.attrs['gidNumber'][0]
-                    is_member = ldap_user.connection.compare_s(force_str(group_dn), 'gidNumber', force_str(user_gid))
+                    is_member = ldap_user.connection.compare_s(
+                        force_text(group_dn),
+                        'gidNumber',
+                        user_gid.encode('utf-8'),
+                    )
                 except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
                     is_member = False
         except (KeyError, IndexError):
@@ -472,9 +473,9 @@ class MemberDNGroupType(LDAPGroupType):
     def is_member(self, ldap_user, group_dn):
         try:
             result = ldap_user.connection.compare_s(
-                force_str(group_dn),
-                force_str(self.member_attr),
-                force_str(ldap_user.dn)
+                force_text(group_dn),
+                force_text(self.member_attr),
+                ldap_user.dn.encode('utf-8'),
             )
         except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE):
             result = 0
@@ -482,43 +483,6 @@ class MemberDNGroupType(LDAPGroupType):
         return result
 
 
-class NISGroupType(LDAPGroupType):
-    """
-    A group type that handles nisNetgroup.
-    """
-    def user_groups(self, ldap_user, group_search):
-        try:
-            user_uid = ldap_user.attrs['uid'][0]
-            filterstr = '(|(nisNetgroupTriple={})(nisNetgroupTriple={}))'.format(
-                self.ldap.filter.escape_filter_chars('(,{},)'.format(user_uid)),
-                self.ldap.filter.escape_filter_chars('(-,{},-)'.format(user_uid))
-            )
-            search = group_search.search_with_additional_term_string(filterstr)
-            groups = search.execute(ldap_user.connection)
-        except (KeyError, IndexError):
-            pass
-        return groups
-
-    def is_member(self, ldap_user, group_dn):
-        try:
-            user_uid = ldap_user.attrs['uid'][0]
-            result = ldap_user.connection.compare_s(
-                force_str(group_dn),
-                force_str('nisNetgroupTriple'),
-                force_str('(,{},)'.format(user_uid))
-            )
-            if result == 0:
-                result = ldap_user.connection.compare_s(
-                    force_str(group_dn),
-                    force_str('nisNetgroupTriple'),
-                    force_str('(-,{},-)'.format(user_uid))
-                )
-        except (ldap.UNDEFINED_TYPE, ldap.NO_SUCH_ATTRIBUTE, KeyError, IndexError):
-            result = 0
-
-        return result
-
-
 class NestedMemberDNGroupType(LDAPGroupType):
     """
     A group type that stores lists of members as distinguished names and
