@@ -1,4 +1,4 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import unicode_literals
 
 from django.contrib.auth.models import AbstractBaseUser
 from django.db import models
